(***********************************************************************)
(*                              Project Couverture                     *)
(*                                                                     *)
(* file: coverageCE.ml                                                 *)
(* authors: Adrien Jonquet, Philippe Wang, Alexis Darrasse             *)
(* licence: CeCIL-B                                                    *)
(***********************************************************************)

type event = EventType.event

let src_list = ref []     
let module_names = ref ([] : string list)

let get_pos event =
  match event.Covevent.covev_kind with
    | Covevent.Coverage_event_before -> event.Covevent.covev_loc.Location.loc_start
    | Covevent.Coverage_event_after _ -> event.Covevent.covev_loc.Location.loc_end
    | _ -> event.Covevent.covev_loc.Location.loc_start
	
let get_loc event =
  event.Covevent.covev_loc.Location.loc_start, event.Covevent.covev_loc.Location.loc_end
    
let relocate_event orig event =
  event.Covevent.covev_pos <- orig + event.Covevent.covev_pos

let get_module_names () =
  let ci_info = open_in "cover.info" in  
    try
      while true do
	module_names := (input_line ci_info) :: !module_names
      done;
    with
      | End_of_file -> () 

let record_events event_array data module_name =
  let ci = open_in (module_name ^ ".cmo.cover") in
  let (orig, ev_list) = Marshal.from_channel ci in
    close_in ci;
    if ev_list = [] then failwith "no coverage_events";
    let event_list_size = List.length ev_list in
      if event_list_size > 0 then
	let rec add src l =
	  match l with
	      [] ->  src_list := src :: !src_list
	    | h :: t -> 
		if src = h then ()
		else add src t
	in
 	  List.iter 
	    (fun event ->
	       relocate_event orig event;
 	       let src = (get_pos event).Lexing.pos_fname in
		 add src !src_list;
		 try 
		   begin
		     match event_array.((event.Covevent.covev_pos) / 4) with
		       | [] -> event_array.((event.Covevent.covev_pos) / 4) <- [event]
		       | l -> event_array.((event.Covevent.covev_pos) / 4) <- event ::l
		   end
		 with
		     Invalid_argument s -> failwith (s^"    wrong orig")
		   | e -> raise e)
	    ((List.fold_left 
		(fun lu event -> 
		   if List.exists 
		     (fun a -> ((fst (get_loc a) = fst (get_loc event))  
				&& (snd (get_loc a) = snd (get_loc event)))) lu
		   then lu else (event :: lu)) [] ev_list))
