(***********************************************************************)
(*                              Project Couverture                     *)
(*                                                                     *)
(* file: pluginTrace.ml                                                *)
(* authors: Adrien Jonquet, Philippe Wang, Alexis Darrasse             *)
(* licence: CeCIL-B                                                    *)
(***********************************************************************)

(* TODO deal with the case of multiple VM instances *)

let trace = ref false
let trace_name = ref ""
let exec_trace = ref (Array.make 0 Trace.No_instr)

let step vm instruction =
  (* record the current obj instruction for source coverage *)
  if !trace then
    Trace.record_instruction vm.Vm.code_pointer !exec_trace instruction

let init data =
  let code_size = Array.length data.Bytecode_loader.code_section in
    (* initialize the execution trace *)
    exec_trace := Array.make code_size Trace.No_instr
      
let finalise () =
  if !trace then
    begin
      (* trace generation *)
      try 
	begin
	  let co = open_out !trace_name in
	    Marshal.to_channel co !exec_trace [];
	    close_out co
	end
      with
	| Sys_error s -> failwith ("pluginTrace.ml : "^s)
    end
      
let _ =
  Plugin.register_plugin init step finalise
    ("-trace", Arg.String (fun s -> trace := true; trace_name := s),
     "file  write trace in file")
