(***********************************************************************)
(*                              Project Couverture                     *)
(*                                                                     *)
(* file: pluginTraceMcdc.ml                                            *)
(* authors: Adrien Jonquet, Mathias Bourgoin                           *)
(* licence: CeCIL-B                                                    *)
(***********************************************************************)

let mcdc_trace_enabled = ref false
let mcdc_trace = ref ([] : (string * (TraceMcdc.mcdc list)) list)
let conds_list = ref ([] : Covevent.coverage_event list)
let dec_list = ref ([] : Covevent.coverage_event list)
let mcdc_trace_name = ref ""
let prims = ref (Array.make 0 "")
    
let init data =
  CoverageUtils.get_module_names ();
  prims := data.Bytecode_loader.primitive_section;
  List.iter (fun module_name ->
	       let l1, l2, l3 = TraceMcdc.build_mcdc_vectors data module_name in
		 mcdc_trace := (module_name, l1) :: !mcdc_trace;
		 conds_list := l2 @ !conds_list;
		 dec_list := l3 @ !dec_list) !CoverageUtils.module_names

let stp = ref 0

let step vm instruction =
  (* record the current obj instruction for source coverage *)
  if !mcdc_trace_enabled then
    ( 
      List.iter 
	(fun (module_name, mcdc_trace_list) ->
	   TraceMcdc.record_mcdc vm mcdc_trace_list !conds_list 
	     !dec_list instruction !prims)				
      !mcdc_trace;
      (* printing vectors *)
      (*incr stp;
      if !stp = 1000 then
	(stp := 0;
	 for i = 0 to List.length !mcdc_trace - 1 do
	   for j = 0 to List.length (snd (List.nth !mcdc_trace i )) - 1 do
	     Printf.printf "%d " (List.length (List.nth (snd (List.nth !mcdc_trace i)) j).TraceMcdc.vectors)
	   done;
	 done;
	 print_newline()
	)*)
    )
   
let finalise () =
  if !mcdc_trace_enabled then
    begin
      (* mcdc trace generation *)
      try
	begin
	  let co = open_out !mcdc_trace_name in
	    Marshal.to_channel co !mcdc_trace [];
	    close_out co
	end
      with
	| Sys_error s -> failwith ("pluginTraceMcdc.ml : "^s)
    end

let _ =
  Plugin.register_plugin init step finalise
    ("-mcdc", Arg.String (fun s -> mcdc_trace_enabled := true; mcdc_trace_name := s),
     "file write mcdc trace in file")
